@@ -36,6 +36,7 @@
 
 import time
 from abc import ABC, abstractmethod
+from typing import Union
 
 from .protocol import *
 from .attributes import *
@@ -93,23 +94,22 @@ class Node(ABC):
         print('[%s] %s' % (time_string, msg))
 
     @abstractmethod
-    def send(self, data: bytes, remote_host: str, remote_port: int) -> int:
+    def send(self, data: bytes, destination: tuple, source: Union[tuple, int] = None) -> int:
         """
         Send data to remote address
 
         :param data:
-        :param remote_host:
-        :param remote_port:
+        :param destination: remote address
+        :param source:      local address
         :return: count of sent bytes
         """
         raise NotImplemented
 
     @abstractmethod
-    def receive(self, buffer_size: int = 2048) -> (bytes, (str, int)):
+    def receive(self) -> (bytes, (str, int)):
         """
         Received data from local port
 
-        :param buffer_size:
         :return: data and remote address
         """
         raise NotImplemented
@@ -188,10 +188,6 @@ class Server(Node, ABC):
         """
         self.another_port: int = 3479
 
-    @abstractmethod
-    def send(self, data: bytes, remote_host: str, remote_port: int, local_port: int=0) -> int:
-        raise NotImplemented
-
     def parse_attribute(self, attribute: Attribute, context: dict, result: Info) -> Info:
         value = attribute.value
         # check attributes
@@ -216,15 +212,13 @@ class Server(Node, ABC):
         :return:
         """
         assert self.neighbour_address is not None, 'neighbour address not set'
-        neighbour_ip = self.neighbour_address[0]
-        neighbour_port = self.neighbour_address[1]
         # create attributes
         value = MappedAddressValue.new(ip=remote_ip, port=remote_port)
         data1 = Attribute(MappedAddress, value).data
         # pack
         body = data1
         pack = Package.new(msg_type=head.type, trans_id=head.trans_id, body=body)
-        self.send(data=pack.data, remote_host=neighbour_ip, remote_port=neighbour_port)
+        self.send(data=pack.data, destination=self.neighbour_address)
 
     def _respond(self, head: Header, remote_ip: str, remote_port: int, local_port: int=0):
         assert self.source_address is not None, 'source address not set'
@@ -255,7 +249,7 @@ class Server(Node, ABC):
         # pack
         body = data1 + data2 + data3 + data4 + data5 + data6
         pack = Package.new(msg_type=BindResponse, trans_id=head.trans_id, body=body)
-        self.send(data=pack.data, remote_host=remote_ip, remote_port=remote_port, local_port=local_port)
+        self.send(data=pack.data, destination=(remote_ip, remote_port), source=(local_ip, local_port))
 
     def handle(self, data: bytes, remote_ip: str, remote_port: int):
         # 1. parse request
@@ -346,39 +340,42 @@ class Client(Node, ABC):
         self.retries = 3
 
     def parse_attribute(self, attribute: Attribute, context: dict, result: Info) -> Info:
-        value = attribute.value
+        a_type = attribute.type
+        a_value = attribute.value
         # check attributes
-        if attribute.type == MappedAddress:
-            assert isinstance(value, MappedAddressValue), 'mapped address value error: %s' % value
-            result.mapped_address = (value.ip, value.port)
-            self.info('MappedAddress:\t(%s:%d)' % (value.ip, value.port))
-        elif attribute.type == XorMappedAddress:
-            if not isinstance(value, XorMappedAddressValue):
+        if a_type == MappedAddress:
+            assert isinstance(a_value, MappedAddressValue), 'mapped address value error: %s' % a_value
+            result.mapped_address = (a_value.ip, a_value.port)
+            self.info('MappedAddress:\t(%s:%d)' % (a_value.ip, a_value.port))
+        elif a_type == XorMappedAddress:
+            if not isinstance(a_value, XorMappedAddressValue):
                 # XOR and parse again
-                data = XorMappedAddressValue.xor(data=value.data, factor=context['trans_id'])
-                value = XorMappedAddressValue.parse(data=data, length=len(data))
-            result.mapped_address = (value.ip, value.port)
-            self.info('XorMappedAddress:\t(%s:%d)' % (value.ip, value.port))
-        elif attribute.type == XorMappedAddress2:
-            if not isinstance(value, XorMappedAddressValue2):
+                data = XorMappedAddressValue.xor(data=a_value.data, factor=context['trans_id'])
+                a_len = AttributeLength(len(data))
+                a_value = XorMappedAddressValue.parse(data=data, t=a_type, length=a_len)
+            result.mapped_address = (a_value.ip, a_value.port)
+            self.info('XorMappedAddress:\t(%s:%d)' % (a_value.ip, a_value.port))
+        elif a_type == XorMappedAddress2:
+            if not isinstance(a_value, XorMappedAddressValue2):
                 # XOR and parse again
-                data = XorMappedAddressValue2.xor(data=value.data, factor=context['trans_id'])
-                value = XorMappedAddressValue2.parse(data=data, length=len(data))
-            result.mapped_address = (value.ip, value.port)
-            self.info('XorMappedAddress2:\t(%s:%d)' % (value.ip, value.port))
-        elif attribute.type == ChangedAddress:
-            assert isinstance(value, ChangedAddressValue), 'changed address value error: %s' % value
-            result.changed_address = (value.ip, value.port)
-            self.info('ChangedAddress:\t(%s:%d)' % (value.ip, value.port))
-        elif attribute.type == SourceAddress:
-            assert isinstance(value, SourceAddressValue), 'source address value error: %s' % value
-            result.source_address = (value.ip, value.port)
-            self.info('SourceAddress:\t(%s:%d)' % (value.ip, value.port))
-        elif attribute.type == Software:
-            assert isinstance(value, SoftwareValue), 'software value error: %s' % value
-            self.info(('Software: %s' % value.description))
+                data = XorMappedAddressValue2.xor(data=a_value.data, factor=context['trans_id'])
+                a_len = AttributeLength(len(data))
+                a_value = XorMappedAddressValue2.parse(data=data, t=a_type, length=a_len)
+            result.mapped_address = (a_value.ip, a_value.port)
+            self.info('XorMappedAddress2:\t(%s:%d)' % (a_value.ip, a_value.port))
+        elif a_type == ChangedAddress:
+            assert isinstance(a_value, ChangedAddressValue), 'changed address value error: %s' % a_value
+            result.changed_address = (a_value.ip, a_value.port)
+            self.info('ChangedAddress:\t(%s:%d)' % (a_value.ip, a_value.port))
+        elif a_type == SourceAddress:
+            assert isinstance(a_value, SourceAddressValue), 'source address value error: %s' % a_value
+            result.source_address = (a_value.ip, a_value.port)
+            self.info('SourceAddress:\t(%s:%d)' % (a_value.ip, a_value.port))
+        elif a_type == Software:
+            assert isinstance(a_value, SoftwareValue), 'software value error: %s' % a_value
+            self.info(('Software: %s' % a_value.description))
         else:
-            self.info('unknown attribute type: %s' % attribute.type)
+            self.info('unknown attribute type: %s' % a_type)
         return result
 
     def __bind_request(self, remote_host: str, remote_port: int, body: bytes) -> Optional[Info]:
@@ -388,7 +385,7 @@ class Client(Node, ABC):
         # 2. send and get response
         count = 0
         while True:
-            size = self.send(data=req.data, remote_host=remote_host, remote_port=remote_port)
+            size = self.send(data=req.data, destination=(remote_host, remote_port))
             if size != len(req.data):
                 # failed to send data
                 return None
